@@ -8,7 +8,7 @@
 
 #include "util.h"
 
-static struct feed **feeds;
+static struct feed *feeds;
 static char *line;
 static size_t linesize;
 static time_t comparetime;
@@ -33,42 +33,42 @@ printfeed(FILE *fpitems, FILE *fpin, struct feed *f)
 		xmlencode(f->name, fpitems);
 		fputs("</a></h2>\n", fpitems);
 	}
+	fputs("<pre>\n", fpitems);
 
 	while ((linelen = getline(&line, &linesize, fpin)) > 0) {
 		if (line[linelen - 1] == '\n')
 			line[--linelen] = '\0';
-		if (!parseline(line, fields))
-			break;
+		parseline(line, fields);
 
 		parsedtime = 0;
-		if (strtotime(fields[FieldUnixTimestamp], &parsedtime))
-			continue;
-		if (!(tm = localtime(&parsedtime)))
-			err(1, "localtime");
-
-		isnew = (parsedtime >= comparetime) ? 1 : 0;
-		totalnew += isnew;
-		f->totalnew += isnew;
+		if (!strtotime(fields[FieldUnixTimestamp], &parsedtime) &&
+		    (tm = localtime(&parsedtime))) {
+			isnew = (parsedtime >= comparetime) ? 1 : 0;
+			totalnew += isnew;
+			f->totalnew += isnew;
+			fprintf(fpitems, "%04d-%02d-%02d&nbsp;%02d:%02d ",
+			        tm->tm_year + 1900, tm->tm_mon + 1, tm->tm_mday,
+			        tm->tm_hour, tm->tm_min);
+		} else {
+			fputs("                 ", fpitems);
+		}
 		f->total++;
 
-		fprintf(fpitems, "%04d-%02d-%02d&nbsp;%02d:%02d ",
-		        tm->tm_year + 1900, tm->tm_mon + 1, tm->tm_mday,
-		        tm->tm_hour, tm->tm_min);
-		if (isnew)
-			fputs("<b><u>", fpitems);
 		if (fields[FieldLink][0]) {
 			fputs("<a href=\"", fpitems);
 			xmlencode(fields[FieldLink], fpitems);
 			fputs("\">", fpitems);
-			xmlencode(fields[FieldTitle], fpitems);
-			fputs("</a>", fpitems);
-		} else {
-			xmlencode(fields[FieldTitle], fpitems);
 		}
+		if (isnew)
+			fputs("<b><u>", fpitems);
+		xmlencode(fields[FieldTitle], fpitems);
 		if (isnew)
 			fputs("</u></b>", fpitems);
+		if (fields[FieldLink][0])
+			fputs("</a>", fpitems);
 		fputs("\n", fpitems);
 	}
+	fputs("</pre>\n", fpitems);
 }
 
 int
@@ -82,7 +82,7 @@ main(int argc, char *argv[])
 	if (pledge("stdio rpath wpath cpath", NULL) == -1)
 		err(1, "pledge");
 
-	if (!(feeds = calloc(argc, sizeof(struct feed *))))
+	if (!(feeds = calloc(argc, sizeof(struct feed))))
 		err(1, "calloc");
 
 	if ((comparetime = time(NULL)) == -1)
@@ -108,29 +108,25 @@ main(int argc, char *argv[])
 	      "\t<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\" />\n"
 	      "\t<link rel=\"stylesheet\" type=\"text/css\" href=\"style.css\" />\n"
 	      "</head>\n"
-	      "<body class=\"frame\"><div id=\"items\"><pre>\n", fpitems);
+	      "<body class=\"frame\"><div id=\"items\">", fpitems);
 
 	if (argc == 1) {
-		if (!(feeds[0] = calloc(1, sizeof(struct feed))))
-			err(1, "calloc");
-		feeds[0]->name = "";
-		printfeed(fpitems, stdin, feeds[0]);
+		feeds[0].name = "";
+		printfeed(fpitems, stdin, &feeds[0]);
 	} else {
 		for (i = 1; i < argc; i++) {
-			if (!(feeds[i - 1] = calloc(1, sizeof(struct feed))))
-				err(1, "calloc");
 			name = ((name = strrchr(argv[i], '/'))) ? name + 1 : argv[i];
-			feeds[i - 1]->name = name;
+			feeds[i - 1].name = name;
 
 			if (!(fp = fopen(argv[i], "r")))
 				err(1, "fopen: %s", argv[i]);
-			printfeed(fpitems, fp, feeds[i - 1]);
+			printfeed(fpitems, fp, &feeds[i - 1]);
 			if (ferror(fp))
 				err(1, "ferror: %s", argv[i]);
 			fclose(fp);
 		}
 	}
-	fputs("</pre>\n</div></body>\n</html>\n", fpitems); /* div items */
+	fputs("</div></body>\n</html>\n", fpitems); /* div items */
 
 	if (showsidebar) {
 		fputs("<!DOCTYPE HTML>\n"
@@ -142,7 +138,7 @@ main(int argc, char *argv[])
 		      "<body class=\"frame\">\n<div id=\"sidebar\">\n", fpmenu);
 
 		for (i = 1; i < argc; i++) {
-			f = feeds[i - 1];
+			f = &feeds[i - 1];
 			if (f->totalnew)
 				fputs("<a class=\"n\" href=\"items.html#", fpmenu);
 			else
