@@ -1,10 +1,7 @@
-#include <sys/types.h>
-
 #include <ctype.h>
 #include <err.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <string.h>
 #include <strings.h>
 
 #include "util.h"
