@@ -93,9 +93,8 @@ encodeuri(char *buf, size_t bufsiz, const char *s)
 	size_t i, b;
 
 	for (i = 0, b = 0; s[i]; i++) {
-		if (s[i] == ' ' ||
-		    (unsigned char)s[i] > 127 ||
-		    iscntrl((unsigned char)s[i])) {
+		if ((unsigned char)s[i] <= ' ' ||
+		    (unsigned char)s[i] >= 127) {
 			if (b + 3 >= bufsiz)
 				return -1;
 			buf[b++] = '%';
@@ -171,13 +170,10 @@ absuri(char *buf, size_t bufsiz, const char *link, const char *base)
 	return encodeuri(buf, bufsiz, tmp);
 }
 
-/* Read a field-separated line from 'fp',
- * separated by a character 'separator',
- * 'fields' is a list of pointers with a size of FieldLast (must be >0).
- * 'line' buffer is allocated using malloc, 'size' will contain the allocated
- * buffer size.
- * returns: amount of fields read (>0) or -1 on error. */
-size_t
+/* Splits fields in the line buffer by replacing TAB separators with NUL ('\0')
+ * terminators and assign these fields as pointers. If there are less fields
+ * than expected then the field is an empty string constant. */
+void
 parseline(char *line, char *fields[FieldLast])
 {
 	char *prev, *s;
@@ -194,8 +190,6 @@ parseline(char *line, char *fields[FieldLast])
 	/* make non-parsed fields empty. */
 	for (; i < FieldLast; i++)
 		fields[i] = "";
-
-	return i;
 }
 
 /* Parse time to time_t, assumes time_t is signed, ignores fractions. */
@@ -247,10 +241,15 @@ printutf8pad(FILE *fp, const char *s, size_t len, int pad)
 
 	slen = strlen(s);
 	for (i = 0; i < slen; i += rl) {
-		if ((rl = mbtowc(&wc, s + i, slen - i < 4 ? slen - i : 4)) <= 0)
-			break;
-		if ((w = wcwidth(wc)) == -1)
+		rl = w = 1;
+		if ((unsigned char)s[i] < 32)
 			continue;
+		if ((unsigned char)s[i] >= 127) {
+			if ((rl = mbtowc(&wc, s + i, slen - i < 4 ? slen - i : 4)) <= 0)
+				break;
+			if ((w = wcwidth(wc)) == -1)
+				continue;
+		}
 		if (col + w > len || (col + w == len && s[i + rl])) {
 			fputs("\xe2\x80\xa6", fp);
 			col++;
